
rm (list = ls (all=TRUE))

library (plm)
library (stats)

alpha <- 0.05

data <- get (data (Grunfeld))

n <- as.numeric (max (data$firm))
t <- as.numeric (max (data$year) - min (data$year)) + 1
init_year <- as.numeric (min (data$year))



procedure_poolability <- function (data, across, include_int) {
  
  if (across == "firm") {
    
    k <- 3
    df_u <- n*(t-k)
    
    urss <- 0
    for (i in 1:n) {
      
      data_select <- data [which (data$firm == i), ]
      unrestricted_model <- lm (inv ~ value + capital, data=data_select)
      urss <- urss + unrestricted_model$residuals %*% unrestricted_model$residuals
    }
  
    if (include_int == 1) {
      
      k <- 3
      df_r <- (n-1)*k   # (nt-k) - n*(t-k) = (n-1)*k
      
      restricted_model <- lm (inv ~ value + capital, data=data)
      rrss <- restricted_model$residuals %*% restricted_model$residuals
    }
    
    if (include_int == 0) {
      
      k <- 2
      df_r <- (n-1)*k
      
      restricted_model <- plm (inv ~ value + capital, data=Grunfeld, model="within")
      rrss <- restricted_model$residuals %*% restricted_model$residuals
    }
  }
  
  if (across == "time") {
    
    k <- 3
    df_u <- t*(n-k)
    
    urss <- 0
    for (j in 1:t) {
      
      data_select <- data [which (data$year == (init_year-1+j)), ]
      unrestricted_model <- lm (inv ~ value + capital, data=data_select)
      urss <- urss + unrestricted_model$residuals %*% unrestricted_model$residuals
    }
    
    if (include_int == 1) {
      
      k <- 3
      df_r <- (t-1)*k
      
      restricted_model <- lm (inv ~ value + capital, data=data)
      rrss <- restricted_model$residuals %*% restricted_model$residuals
    }
    
    if (include_int == 0) {
      
      k <- 2
      df_r <- (t-1)*k
      
      restricted_model <- plm (inv ~ value + capital, data=data, model="within")
      rrss <- restricted_model$residuals %*% restricted_model$residuals
    }
  }
  
  f_obs <- (rrss-urss)*df_u/(urss*df_r)
  
  # results

  results <- round (c (rrss, urss, f_obs, df_r, df_u, qf (1-alpha, df1=df_r, df2=df_u)), 2)
  names (results) <- c ("RRSS", "URSS", "F_obs", "df1", "df2", "q")
  results
}



##### poolability across data

# across firms

  # including intercept
  procedure_poolability (data, across="firm", include_int=1)
  # excluding intercept
  procedure_poolability (data, across="firm", include_int=0)

# acorss time

  # including intercept
  procedure_poolability (data, across="time", include_int=1)
  # excluding intercept
  procedure_poolability (data, across="time", include_int=0)


##### tests for individual and time effects

  vec_effects <- c ("individual", "time", "twoways")
  vec_type <- c ("bp", "honda", "kw", "ghm", "f")

  results_effects <- matrix (0, nrow=5, ncol=3)
  colnames (results_effects) <- vec_effects
  rownames (results_effects) <- vec_type

  critical_values <- matrix (0, nrow=5, ncol=3)
  colnames (critical_values) <- vec_effects
  rownames (critical_values) <- vec_type

  plm_model <- plm (inv ~ value + capital, data=data, model="pooling")
    
  for (j in 1:2) { 
    for (i in 1:3) {
    
      results_effects [j, i] <- plmtest (plm_model, effect=vec_effects [i], type=vec_type [j])$statistic
    }
  }

  for (j in 3:4) {  
    
    results_effects [j, 3] <- plmtest (plm_model, effect=vec_effects [3], type=vec_type [j])$statisti
  }

  results_effects [5, 3] <- pFtest (inv ~ value + capital, data=data, effect="twoways")$statistic

  critical_values [1, 1:2] <- qchisq (1-alpha, df=1)
  critical_values [1, 3] <- qchisq (1-alpha, df=2)
  critical_values [2, 1:3] <- qnorm (1-alpha)
  critical_values [3, 3] <- qnorm (1-alpha)
  critical_values [4, 3] <- qchisq (1-alpha, df=1)/2 + qchisq (1-alpha, df=2)/4
  critical_values [5, 3] <- qf (1-alpha, df1=3*(n-1), df2=n*t-3)

  results_effects
  critical_values


##### Hausman's specificarion test

  results_hausman <- rep (0, 3)
  names (results_hausman) <- c ("m_1", "m_2", "m_3")

  model_within <- plm (inv ~ value + capital, data=data, effect="individual", model="within")
  model_between <- plm (inv ~ value + capital, data=data, effect="individual", model="between")
  model_gls <- plm (inv ~ value + capital, data=data, model="random")

  results_hausman [1] <- phtest (model_within, model_gls)$statistic
  results_hausman [2] <- phtest (model_between, model_gls)$statistic
  results_hausman [3] <- phtest (model_within, model_between)$statistic

  results_hausman
  qchisq (1-alpha, df=2)


##### estimating of regression parameters

  plm (inv ~ value + capital, data=data, model="random")$coefficient


